import json
from base64 import b64encode
from typing import List

import respx
from django.contrib.auth import get_user_model
from django.contrib.auth.models import AbstractUser, Permission
from django.test import TestCase, override_settings
from django.urls import reverse
from ferda.tests.utils import LOGGER_SESSION_ID, LoggerMixin
from httpx import Response

from ferda_auction.constants import ApiScope

User = get_user_model()


@override_settings(
    ROOT_URLCONF="ferda_auction.tests.urls",
    FERDA_AUCTION_API_URL="https://auctions.example.com/api/",
    FERDA_AUCTION_API_USERNAME="rimmer",
    FERDA_AUCTION_API_PASSWORD="Gazpacho!",
)
class GetAccessTokenTest(LoggerMixin, TestCase):
    @respx.mock
    def _test_access_token(
        self, user: AbstractUser, permissions: List[str], scopes: List[ApiScope]
    ) -> None:
        user.user_permissions.set(
            Permission.objects.filter(
                codename__in=permissions, content_type__model="ferda_auction"
            )
        )
        self.client.force_login(user)
        respx.post("https://auctions.example.com/api/v1/auth/login/admin/").mock(
            return_value=Response(200, json={"access_token": "TEST_TOKEN"})
        )

        response = self.client.get(reverse("ferda_auction:api:get_access_token"))

        self.assertEqual(response.json(), {"access_token": "TEST_TOKEN"})
        self.assertEqual(
            json.loads(respx.calls.last.request.content),
            {
                "logger_session_id": LOGGER_SESSION_ID,
                "scopes": scopes,
            },
        )
        self.assertEqual(
            respx.calls.last.request.headers["Authorization"],
            "Basic " + b64encode(b"rimmer:Gazpacho!").decode(),
        )

    def test_view(self):
        user = User.objects.create_user("rimmer")
        self._test_access_token(user, ["can_view"], [])

    def test_view_all(self):
        user = User.objects.create_user("rimmer")
        self._test_access_token(user, ["can_view", "can_view_all"], [ApiScope.AUCTION_READ_FULL])

    def test_edit_auction(self):
        user = User.objects.create_user("rimmer")
        self._test_access_token(user, ["can_view", "can_edit_auction"], [ApiScope.AUCTION_EDIT])

    def test_edit_ban(self):
        user = User.objects.create_user("rimmer")
        self._test_access_token(
            user, ["can_view", "can_edit_ban"], [ApiScope.BAN_READ, ApiScope.BAN_EDIT]
        )

    def test_superuser(self):
        user = User.objects.create_superuser("admin")
        scopes = [
            ApiScope.AUCTION_READ_FULL,
            ApiScope.AUCTION_EDIT,
            ApiScope.BAN_READ,
            ApiScope.BAN_EDIT,
        ]
        self._test_access_token(user, [], scopes)

    @override_settings(FERDA_AUCTION_API_USERNAME=None, FERDA_AUCTION_API_PASSWORD=None)
    @respx.mock
    def test_no_auth(self):
        user = User.objects.create_superuser("admin")
        self.client.force_login(user)

        respx.post("https://auctions.example.com/api/v1/auth/login/admin/").mock(
            return_value=Response(200, json={"access_token": "TEST_TOKEN"})
        )

        response = self.client.get(reverse("ferda_auction:api:get_access_token"))

        self.assertEqual(response.json(), {"access_token": "TEST_TOKEN"})
        self.assertEqual(
            json.loads(respx.calls.last.request.content),
            {
                "logger_session_id": LOGGER_SESSION_ID,
                "scopes": [
                    ApiScope.AUCTION_READ_FULL,
                    ApiScope.AUCTION_EDIT,
                    ApiScope.BAN_READ,
                    ApiScope.BAN_EDIT,
                ],
            },
        )
        self.assertNotIn("Authorization", respx.calls.last.request.headers)
